@@ -2,32 +2,62 @@ import json
 import random
 import re
 import string
+import os
 
 import pypandoc
 
+from tqdm import tqdm
+
 html_convert = lambda x: pypandoc.convert_text(x, "html5", "md", ["--mathjax"])
 tex_convert = lambda x: pypandoc.convert_text(x, "latex", "md")
 
 
 class LineBuffer:
     def __init__(self, text):
-        self.lines = text.strip().split("\n")
+        self.lines = []
         self.i = 0
+        self.insert_next(text)
+        self.onpop = lambda x: None
+
+    def insert_next(self, text):
+        if isinstance(text, list):
+            new_lines = text
+        elif isinstance(text, str):
+            new_lines = text.strip().split("\n")
+        elif isinstance(text, LineBuffer):
+            new_lines = text.lines
+            self.i += text.i
+        else:
+            raise SyntaxError(f"LineBuffer: unsupported type to insert: {type(text)}")
+        self.lines = self.lines[: self.i] + new_lines + self.lines[self.i :]
 
     def pop(self) -> str:
         if self.i == len(self.lines):
             raise SyntaxError("File terminated unexpectedly")
         self.i += 1
+        self.onpop(self)
         return self.lines[self.i - 1]
 
+    def remove_prev(self):
+        self.i -= 1
+        if self.i < 0:
+            self.i = 0
+        del self.lines[self.i]
+
     def empty(self):
         return self.i == len(self.lines)
 
+    def reset(self):
+        self.i = 0
+
+    def location(self):
+        return self.i
+
 
 def parse_directive(line):
     if not any(
-            line.startswith(f"# {x} ")
-            for x in ["BEGIN", "END", "INPUT", "CONFIG", "DEFINE"]
+        line.startswith(f"# {x} ")
+        for x in ["BEGIN", "END", "INPUT", "CONFIG", "DEFINE", "IMPORT"]
     ):
         return None, None, None
     tokens = line.split(" ", 3)
@@ -57,7 +87,7 @@ def rand_id():
     return "".join(random.choices(string.ascii_uppercase, k=32))
 
 
-class ToParse():
+class ToParse:
     def __init__(self, text, type):
         self.text = text
         self.type = type
@@ -67,11 +97,7 @@ class ToParse():
 
 
 def parse(text):
-    return {
-        "text": text,
-        "html": html_convert(text),  # ToParse(text, "html"),
-        "tex": tex_convert(text)  # ToParse(text, "tex"),
-    }
+    return {"text": text, "html": ToParse(text, "html"), "tex": ToParse(text, "tex")}
 
 
 def parse_define(directive, rest, substitutions, substitutions_match):
@@ -109,13 +135,13 @@ def parse_input_lines(lines):
             is_fixed = False
             if rest.startswith(fixed):
                 is_fixed = True
-                rest = rest[len(fixed):]
+                rest = rest[len(fixed) :]
 
             correct = "CORRECT "
             is_correct = False
             if rest.startswith(correct):
                 is_correct = True
-                rest = rest[len(correct):]
+                rest = rest[len(correct) :]
 
             if is_correct:
                 correct_options.append(rest)
@@ -128,10 +154,10 @@ def parse_input_lines(lines):
             correct_options,
         )
     elif directive in (
-            "SHORT_ANSWER",
-            "SHORT_CODE_ANSWER",
-            "LONG_ANSWER",
-            "LONG_CODE_ANSWER",
+        "SHORT_ANSWER",
+        "SHORT_CODE_ANSWER",
+        "LONG_ANSWER",
+        "LONG_CODE_ANSWER",
     ):
         if len(lines) > 1:
             raise SyntaxError(
@@ -163,9 +189,13 @@ def consume_rest_of_solution(buff, end):
             if directive == end:
                 return parse("\n".join(out))
             else:
-                raise SyntaxError("Unexpected END in SOLUTION")
+                raise SyntaxError(
+                    f"Unexpected END ({directive if directive else line}) in SOLUTION"
+                )
         else:
-            raise SyntaxError("Unexpected directive in SOLUTION")
+            raise SyntaxError(
+                f"Unexpected directive ({mode if mode else line}) in SOLUTION"
+            )
 
 
 def consume_rest_of_question(buff):
@@ -192,7 +222,9 @@ def consume_rest_of_question(buff):
             elif directive == "NOTE":
                 solution_note = consume_rest_of_solution(buff, directive)
             else:
-                raise SyntaxError("Unexpected BEGIN in QUESTION")
+                raise SyntaxError(
+                    f"Unexpected BEGIN ({directive if directive else line}) in QUESTION"
+                )
         elif mode == "END":
             if directive == "QUESTION":
                 question_type, options, option_solutions = parse_input_lines(
@@ -217,13 +249,17 @@ def consume_rest_of_question(buff):
                     "substitutions_match": substitutions_match,
                 }
             else:
-                raise SyntaxError("Unexpected END in QUESTION")
+                raise SyntaxError(
+                    f"Unexpected END {directive if directive else line} in QUESTION"
+                )
         elif mode == "DEFINE":
             parse_define(directive, rest, substitutions, substitutions_match)
         elif mode == "CONFIG":
             config[directive] = rest
         else:
-            raise SyntaxError("Unexpected directive in QUESTION")
+            raise SyntaxError(
+                f"Unexpected directive ({mode if mode else line}) in QUESTION"
+            )
 
 
 def consume_rest_of_group(buff, end):
@@ -284,20 +320,23 @@ def consume_rest_of_group(buff, end):
             elif directive == "SCRAMBLE":
                 scramble = True
             else:
-                raise SyntaxError("Unexpected CONFIG directive in GROUP")
+                raise SyntaxError(
+                    f"Unexpected CONFIG directive ({directive if directive else line}) in GROUP"
+                )
         else:
-            raise SyntaxError("Unexpected directive in GROUP")
+            raise SyntaxError(f"Unexpected directive ({line}) in GROUP")
 
 
-def _convert(text):
+def _convert(text, *, path=None):
     buff = LineBuffer(text)
     groups = []
     public = None
     config = {}
     substitutions = {}
     substitutions_match = []
-
     try:
+        if path is not None:
+            handle_imports(buff, path)
         while not buff.empty():
             line = buff.pop()
             if not line.strip():
@@ -331,9 +370,11 @@ def _convert(text):
             elif mode == "DEFINE":
                 parse_define(directive, rest, substitutions, substitutions_match)
             else:
-                raise SyntaxError("Unexpected directive")
+                raise SyntaxError(f"Unexpected directive: {line}")
     except SyntaxError as e:
-        raise SyntaxError("Parse stopped on line {} with error {}".format(buff.i, e))
+        raise SyntaxError(
+            "Parse stopped on line {} with error {}".format(buff.location(), e)
+        )
 
     return {
         "public": public,
@@ -344,7 +385,7 @@ def _convert(text):
     }
 
 
-def pandoc(target):
+def pandoc(target, *, draft=False):
     to_parse = []
 
     def explore(pos):
@@ -361,18 +402,24 @@ def pandoc(target):
 
     DELIMITER = """\n\nDELIMITER\n\n"""
 
-    transpile_target = lambda t: DELIMITER.join(x.text for x in to_parse if x.type == t)
+    if draft:
+        transpile_target = lambda t: DELIMITER.join(
+            x.text for x in to_parse if x.type == t
+        )
 
-    html = html_convert(transpile_target("html")).split(html_convert(DELIMITER))
-    tex = tex_convert(transpile_target("tex")).split(tex_convert(DELIMITER))
+        html = html_convert(transpile_target("html")).split(html_convert(DELIMITER))
+        tex = tex_convert(transpile_target("tex")).split(tex_convert(DELIMITER))
 
-    assert not html or not tex or (len(to_parse) == len(html) + len(tex), (len(to_parse), len(html), len(tex)))
+        for x, h in zip(filter(lambda x: x.type == "html", to_parse), html):
+            x.html = h
 
-    for x, h in zip(filter(lambda x: x.type == "html", to_parse), html):
-        x.html = h
-
-    for x, t in zip(filter(lambda x: x.type == "tex", to_parse), tex):
-        x.tex = t
+        for x, t in zip(filter(lambda x: x.type == "tex", to_parse), tex):
+            x.tex = t
+    else:
+        for x in tqdm(to_parse):
+            x.__dict__[x.type] = (
+                html_convert(x.text) if x.type == "html" else tex_convert(x.text)
+            )
 
     def pandoc_dump(obj):
         assert isinstance(obj, ToParse)
@@ -381,9 +428,32 @@ def pandoc(target):
     return json.dumps(target, default=pandoc_dump)
 
 
-def convert(text):
-    return json.loads(convert_str(text))
+def convert(text, *, path=None, draft=False):
+    return json.loads(convert_str(text, path=path, draft=draft))
 
 
-def convert_str(text):
-    return pandoc(_convert(text))
+def convert_str(text, *, path=None, draft=False):
+    return pandoc(_convert(text, path=path), draft=draft)
+
+
+def import_file(filepath: str) -> str:
+    if not filepath:
+        raise SyntaxError("IMPORT must take in a filepath")
+    with open(filepath, "r") as f:
+        return f.read()
+
+
+def handle_imports(buff: LineBuffer, path: str):
+    while not buff.empty():
+        line = buff.pop()
+        mode, directive, rest = parse_directive(line)
+        if mode == "IMPORT":
+            buff.remove_prev()
+            filepath = " ".join([directive, rest]).rstrip()
+            if path:
+                filepath = os.path.join(path, filepath)
+            new_buff = LineBuffer(import_file(filepath))
+            folderpath = os.path.dirname(filepath)
+            handle_imports(new_buff, folderpath)
+            buff.insert_next(new_buff)
+    buff.reset()
