@@ -29,11 +29,8 @@ def scramble(email, exam, *, keep_data=False):
                 scramble_question(
                     element, [*substitutions, group_substitutions], config
                 )
-        if (
-            group.get("pick_some") == 1
-            and not group["name"].strip()
-            and group["points"] is None
-        ):
+
+        if is_compressible_group(group):
             text, html, tex = group["text"], group["html"], group["tex"]
             element = get_elements(group)[0]
             if element.get("type") != "group" and depth == 1:
@@ -52,8 +49,9 @@ def scramble(email, exam, *, keep_data=False):
         substitute(
             question, [question_substitutions, *substitutions], ["html", "tex", "text"]
         )
-        if "scramble_options" in config and isinstance(question["options"], list):
-            scramble_keep_fixed(question["options"])
+        if isinstance(question["options"], list):
+            if "scramble_options" in config:
+                scramble_keep_fixed(question["options"])
             for option in question["options"]:
                 substitute(
                     option,
@@ -67,13 +65,15 @@ def scramble(email, exam, *, keep_data=False):
                 substitute(
                     solution["solution"], [question_substitutions, *substitutions], ["html", "tex", "text"], store=False
                 )
-            elif solution["options"] is not None:
-                substitute(
-                    solution["options"],
-                    [question_substitutions, *substitutions],
-                    range(len(solution["options"])),
-                    store=False,
-                )
+            else:
+                options = solution["options"]
+                if options:
+                    substitute(
+                        options,
+                        [question_substitutions, *substitutions],
+                        range(len(options)),
+                        store=False,
+                    )
         else:
             question.pop("solution", None)
 
@@ -85,12 +85,27 @@ def scramble(email, exam, *, keep_data=False):
                 for k, v in substitutions.items():
                     target[attr] = target[attr].replace(k, v)
                     target[attr] = target[attr].replace(k.title(), v.title())
+                    target[attr] = target[attr].replace(latex_escape(k), latex_escape(v))
         if store:
             if keep_data:
                 target["substitutions"] = merged
             else:
                 target.pop("substitutions", None)
 
+    def scramble_keep_fixed(objects):
+        if keep_data:
+            for i, object in enumerate(objects):
+                object["index"] = i
+        movable_object_pos = []
+        movable_object_values = []
+        for i, object in enumerate(objects):
+            if not object.get("fixed"):
+                movable_object_pos.append(i)
+                movable_object_values.append(object)
+        random.shuffle(movable_object_values)
+        for i, object in zip(movable_object_pos, movable_object_values):
+            objects[i] = object
+
     global_substitutions = select(exam["substitutions"])
     global_substitutions.update(select_no_replace(exam.get("substitutions_match", [])))
     exam["config"]["scramble_groups"] = exam["config"].get(
@@ -105,18 +120,6 @@ def scramble(email, exam, *, keep_data=False):
     return exam
 
 
-def scramble_keep_fixed(objects):
-    movable_object_pos = []
-    movable_object_values = []
-    for i, object in enumerate(objects):
-        if not object.get("fixed"):
-            movable_object_pos.append(i)
-            movable_object_values.append(object)
-    random.shuffle(movable_object_values)
-    for i, object in zip(movable_object_pos, movable_object_values):
-        objects[i] = object
-
-
 def get_elements(group):
     return group.get("elements") if "elements" in group else group.get("questions")
 
@@ -141,3 +144,11 @@ def select_no_replace(substitutions_match):
             values.remove(c)
             out[choice] = c
     return out
+
+
+def is_compressible_group(group):
+    return group.get("pick_some") == 1 and not group["name"].strip() and group["points"] is None
+
+
+def latex_escape(text):
+    return text.replace("_", r"\_")
